#!/usr/bin/env python
# coding: utf-8
"""
Usage: chanjo-autopilot.py <bam_path> <sql_path> [--cutoff=<kn>]
       chanjo-autopilot.py new <bam_path> <sql_path> <ccds_path> [--cutoff=<kn>]
       chanjo-autopilot.py -h | --help
       chanjo-autopilot.py --version

Process all exons in the SQLite database, get coverage from BAM alignment and
commit changes to SQLite again.

Arguments:
  <bam_path> path to the BAM alignment file
  <sql_path> path to the SQLite database file
  <ccds_path> path to the CCDS database dump

Options:
  -h --help   Show this screen.
  --version   Show version.
"""
from docopt import docopt
import time
import sys
sys.path.append("/Users/robinandeer/SciLife/modules/chanjo2")

from chanjo import core, sql, bam, ccds2sql

def main(args):
  start = time.time()

  # We can set up a brand new database
  if args["new"]:
    imp = ccds2sql.Importer(args["<sql_path>"], args["<ccds_path>"])
    imp.populate()

  cov = bam.CoverageAdapter(args["<bam_path>"])
  db = sql.ElementAdapter(args["<sql_path>"])
  hub = core.Hub(cov, db)

  genes = hub.db.get("gene")
  for gene in genes:
    # Annotate the gene
    hub.annotate(gene, args["--cutoff"])

  # Persist all annotations
  hub.db.commit()

  end = time.time()
  print "Elapsed time: {nosec} seconds".format(nosec=(end-start))

if __name__ == '__main__':
  # parse arguments based on docstring above
  arguments = docopt(__doc__, version='Chanjo 0.0.1')

  main(arguments)



from flask import Flask, jsonify
from sql import ElementAdapter
from datetime import timedelta
from flask import make_response, request, current_app
from functools import update_wrapper

app = Flask(__name__)
adapter = ElementAdapter("tests/data/CCDS_test.db")

def crossdomain(origin=None, methods=None, headers=None, max_age=21600,
                attach_to_all=True, automatic_options=True):

  if methods is not None:
    methods = ', '.join(sorted(x.upper() for x in methods))
  if headers is not None and not isinstance(headers, basestring):
    headers = ', '.join(x.upper() for x in headers)
  if not isinstance(origin, basestring):
    origin = ', '.join(origin)
  if isinstance(max_age, timedelta):
    max_age = max_age.total_seconds()

  def get_methods():
    if methods is not None:
      return methods

    options_resp = current_app.make_default_options_response()
    return options_resp.headers['allow']

  def decorator(f):
    def wrapped_function(*args, **kwargs):
      if automatic_options and request.method == 'OPTIONS':
        resp = current_app.make_default_options_response()
      else:
        resp = make_response(f(*args, **kwargs))
      if not attach_to_all and request.method != 'OPTIONS':
        return resp

      h = resp.headers

      h['Access-Control-Allow-Origin'] = origin
      h['Access-Control-Allow-Methods'] = get_methods()
      h['Access-Control-Max-Age'] = str(max_age)
      if headers is not None:
        h['Access-Control-Allow-Headers'] = headers
      return resp

    f.provide_automatic_options = False
    return update_wrapper(wrapped_function, f)
  return decorator

# =============================================================================
#   Routes
# -----------------------------------------------------------------------------
@app.route("/genes")
@crossdomain(origin='*')
def genes():
  genes = adapter.get("gene")[:20]

  res = {
    "genes": [gene.toDict() for gene in genes]
  }

  return jsonify(**res)

@app.route("/genes/<gene_id>")
@crossdomain(origin='*')
def gene(gene_id):
  gene = adapter.get("gene", gene_id)

  res = {
    "gene": gene.toDict(),
    "transcripts": [tx.toDict() for tx in gene.transcripts],
    "exons": [ex.toDict() for ex in gene.exons]
  }

  return jsonify(**res)

@app.route("/transcripts/<tx_id>")
@crossdomain(origin='*')
def transcripts(tx_id):
  tx = adapter.get("transcript", tx_id)

  res = {
    "transcript": tx.toDict(),
    "gene": tx.gene.toDict(),
    "exons": [ex.toDict() for ex in tx.exons]
  }

  return jsonify(**res)

@app.route("/exons/<exon_id>")
@crossdomain(origin='*')
def exons(exon_id):
  exon = adapter.get("exon", exon_id)

  res = {
    "exon": exon.toDict(),
    "genes": [gene.toDict() for gene in exon.genes],
    "transcripts": [tx.toDict() for tx in exon.transcripts]
  }

  return jsonify(**res)

if __name__ == "__main__":
    app.run(debug=True)